{{

 SpaceInvaders Emulator
 by Chris Cantrell
 6/23/2011
 Copyright (c) 2011 Chris Cantrell
 See end of file for terms of use.

 A note on ROMS:

 This project does not contain the Space Invaders ROM. ROM images are
 available on the web, but you must own the Space Invaders boards
 to legally download them.

 Format an SD card and copy the single binary image to the SD card's
 root directory.           

 **SOUND**
 Port pins P4 and P5 are the audio output (only 5 is used).

 **SD CARD**
 P6 SD_D0     Pulled up to 3.3 with 10K
 P7 SD_CLK
 P8 SD_DI
 P9 SD_CS

 **UNUSED**
 P0,P1,P2,P3,P4,  P10,P11,P12,P13,  P22,P23
 
 **SWITCHES (all pulled up to 3.3 with 10K)**
 P14     Left
 P15     Fire
 P16     Down
 P17     Right
 P18     Start
 P19     Select
 P20     Reset
 P21     Up

 **VIDEO (3 bit DAC from demo schematic)**
 P24  1.1K
 P25  560
 P26  270

 **LED**
 P27  240 ohm resistor    
 
}}

CON      
        _STACK      = 200
        _clkmode    = xtal1 + pll16x
        _xinfreq    = 5_000_000
                              
OBJ
' disk   : "Disk_hdw"           ' SD card driver - DISABLED, ROM is inlined!
  ports  : "SIPorts"            ' Emulates the I/O ports of the SpaceInvaders hardware
  video  : "SIVideo"            ' NTSC Video generator based on SpaceInvaders hardware
  cpu    : "CPU_8080"           ' 8080 CPU emulator
  snd    : "SISound"            ' Produces the discrete SpaceInvaders sounds
  pad    : "PadWrap"
' dbg    : "FullDuplexSerial"

VAR
{
  ' Disk controller
  long diskCommand
  long diskSectorAddress
  long diskMemoryAddress
}
  ' Shared between CPU and PORTS
  long portCommand
  long portAddress
  long portValue

  ' Shared between PORTS and SND
  long soundValue

  ' Shared between PORTS and VIDEO
  long rowNum 

  ' Shared between PORTS and CPU
  long interrupt
{
  ' Used by CPU and VIDEO (screen memory)
  byte rom[8*1024]            '0000-1FFF
  byte ram[1*1024]            '2000-23FF
  byte screenMemory[7*1024]   '2400-3FFF
  byte rom2[2*1024]           ' Space Invaders 2 has an extra ROM chip
}
  ' Parameters for the video COG
  long v_scrPtr
  long v_rowNum

  ' Parameters for the CPU
  long p_memory_base     
  long p_io_base    
  long p_interrupt              

  ' Parameters for the PORTS
  long o_portOp
  long o_rowNum
  long o_soundValue
  long o_interrupt

  'long debugPtr         ' For debugging to the SI screen
  
PUB start | i,p,cc
  ' debugPtr := SCREEN
{
  ' Start disk controller and mount the card
  diskCommand := 1
  diskMemoryAddress := @rom[0] ' Memory area needed by the mount
  disk.start(5,@diskCommand)  
  repeat while diskCommand<>0
  ' Read 16 chunks (512*36 = 18K)
  diskSectorAddress := 0  
  diskCommand := $1_24
  diskMemoryAddress := @rom[0]
  repeat while diskCommand<>0   
}
'  pad.start
  pad.init
  dbg.start(31, 30, 0, 115200)

  portCommand := 0
  rowNum := 0
  soundValue :=0
  interrupt := 0

  o_portOp := @portCommand
  o_rowNum := @rowNum
  o_soundValue := @soundValue
  o_interrupt := @interrupt
  ports.start(6,@o_portOp)

  ' SI display is 224 rows
  ' NTCS display gives us 208 rows
  ' 224 - 208 = 16 less rows that original SI
  ' We'll divide those into 8 on each side
  ' The SI game stays within this range
  v_scrPtr := @screenMemory + 256
  v_rowNum := @rowNum       
  video.start(7,@v_scrPtr)

  p_memory_base := @rom 
  p_io_base := @portCommand  
  p_interrupt := @interrupt   
  cpu.start(@p_memory_base)

'  snd.Main(@soundValue)
  snd.start(@soundValue)

  pad.loop
{
 
PUB writePort(address,value)

  portAddress := address
  portValue := value
  portCommand := 1
  repeat while (portCommand<>0)

PUB readPort(address)
  portAddress := address
  portCommand :=2
  repeat while (portCommand<>0)
  return portValue

PUB debugDigit(value) | p, i
    
  value := value & %1111  
  if value>9
    p := SI_CHARS
    value := value - 10
  else
    p := SI_DIGS
    
  value := value *5
  p := p + value

  repeat i from 1 to 5    
    ram[debugPtr] := rom[p]  
    debugPtr := debugPtr + 32
    p := p + 1

  debugPtr := debugPtr + 32*3  

PUB debugByte(value) | a
  a := value >> 4
  debugDigit(a)
  debugDigit(value)  

PUB debugWord(value) | a
  a := value >>8
  debugByte(a)
  debugByte(value)
}

DAT
rom           file      "invaders.rom"          '0000-1FFF
ram           byte      0[1*1024]               '2000-23FF
screenMemory  byte      0[7*1024]               '2400-3FFF
rom2          byte      0[2*1024]               ' Space Invaders 2 has an extra ROM chip

{{
┌─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┐
│                                                   TERMS OF USE: MIT License                                                 │                                                            
├─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤
│Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation   │ 
│files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,   │
│modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the        │
│Software is furnished to do so, subject to the following conditions:                                                         │         
│                                                                                                                             │
│The above copyright notice and this permission notice shall be included in all copies or substantial portions of the         │
│Software.                                                                                                                    │
│                                                                                                                             │
│THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE         │
│WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR        │
│COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,  │
│ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                        │
└─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘
}}  
